# -*- coding: utf-8 -*-
"""
smash.core
~~~~~~~~~~

Core functionality for Smash.
"""

import os


class Smash(object):
    """Main class for Smash"""

    def __init__(self):
        self.MOVE = {
            'down' : ':',
            'none' : '+',
            'up'   : ']',
        }   

    def set_custom_mov(self, new_mov):
        """Set custom movements"""

        movs = new_mov.split(',')
        if len(movs) == 3 and all([len(c) == 1 for c in movs]):
            self.MOVE['down'] = movs[0]
            self.MOVE['none'] = movs[1]
            self.MOVE['up'] = movs[2]
            return True
        else:
            return False


    def tokens(self, str):
        """Extract all tokens from the given string.

        A token is:
        - a word who not contain the characters in the attribute self.MOVE
        - every character contained in the attribute self.MOVE
        """

        token = ""

        for i in range(len(str) + 1):

            if i < len(str):
                is_flag = str[i] in self.MOVE.values()
                end_string = i == len(str) - 1

                _movs = [self.MOVE['down'], self.MOVE['none']]
                if ((is_flag and not token) or (is_flag and end_string)
                    or (str[i] in _movs and token in self.MOVE.values())
                    or (is_flag and token in _movs)):
                    raise ValueError
                elif is_flag or token in self.MOVE.values():
                    yield token
                    token = str[i]
                else:
                    token += str[i]
            else:
                yield token


    def paths(self, str, topdir):
        """Generate all the paths"""

        stack = []
        stack.append(topdir)

        path = ""

        for token in self.tokens(str):
            if token not in self.MOVE.values():
                path = os.path.join(stack[-1], token)
                yield path
            elif token == self.MOVE['down']:
                    stack.append(path)
            elif token == self.MOVE['up'] and len(stack) > 1:
                    stack.pop()
            else:
                pass  # token == self.MOVE['none'] or token == "":


    def make_dirtree(self, str, topdir):
        """Creates all the directories"""

        done = []
        not_done = []

        _paths = [p for p in self.paths(str, topdir)]
        for p in _paths:
            try:
                os.makedirs(p)
                done.append(p)
            except OSError:
                not_done.append(p)

        return done, not_done


    # functions for printing the preview of the directory tree
    # ========================================================================

    def has_sibling(self, rest):
        """Check there is a sibling"""

        sum = 0
        for i, t in enumerate(rest):

            if t == self.MOVE['down']:
                sum += 1
            elif t == self.MOVE['up']:
                sum -= 1

            if sum < 0:
                return False

            if sum == 0:
                return True

        return False


    def make_spacer(self, vert_links, width):
        """Construct the space"""

        spacing = ""
        for i in range(width):
            if i in vert_links:
                spacing += '│'
            else:
                spacing += ' '

        return spacing


    def print_dirtree(self, str):
        """Print the directory tree"""

        stack_ws = []
        default_spacing = 4
        spacing = 0
        vert_links = []

        ts = [t for t in self.tokens(str)]

        print('.')
        for i, token in enumerate(ts):

            if token not in self.MOVE.values():
                spacer = self.make_spacer(vert_links, spacing)
                conn = ""
                if i + 1 < len(ts) and self.has_sibling(ts[i + 1:]):
                    if spacing not in vert_links:
                        vert_links.append(spacing)
                    conn = "├── "
                else:
                    if spacing in vert_links:
                        vert_links.remove(spacing)
                    conn = "└── "

                print(spacer + conn + token)

            elif token == self.MOVE['down']:
                stack_ws.append(spacing)
                spacing += default_spacing
            elif token == self.MOVE['up'] and len(stack_ws) > 0:
                spacing = stack_ws.pop()
